#
# Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

import random
import signal
import time
from abc import ABCMeta, abstractmethod

import yaml

from netq_lib.orm.redisdb.models import WjhDropAggregate, WjhL1DropAggregate



class BaseWjhClient(object, metaclass=ABCMeta):
    def __init__(self, logger):
        self._logger = logger
        self.agg_cache = {}
        self.running = False
        signal.signal(signal.SIGTERM, self.sig_handler)
        signal.signal(signal.SIGINT, self.sig_handler)

    def flush_agg_cache(self):
        for key, agg in list(self.agg_cache.items()):
            self._logger.debug("Aggregate type: {}, count: {} first {} last {}".format(agg.drop_type, agg.agg_count, agg.first_timestamp, agg.last_timestamp))
            agg.save()

        self.agg_cache.clear()
        mcls = WjhDropAggregate.memmodel()
        mcls.flush()

    @abstractmethod
    def start_wjh_client(self):
        raise NotImplementedError

    def sig_handler(self, signum, frame):
        if self.running:
            self._logger.info("Interrupt/Terminate received "
                              "for wjh process, terminating process")
            self.running = False
