#
# Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from netq_agent.parsers.nvos.redis_db.client import NvosDBRender
from netq_agent.parsers.nvos.redis_db.nvos_db import NvosDB, NvosTable
from netq_lib.orm.redisdb.models import IbInterface

PLR_BW_LOSS_PERCENT_DIVISOR = 1e8
class IbInterfaceHandler(NvosDBRender):
    MODEL = IbInterface

    def __init__(self):
        super(IbInterfaceHandler, self).__init__()

    def db_sources(self):
        return {
            (NvosDB.COUNTERS_DB, NvosTable.COUNTERS_DB_COUNTERS_PORT_NAME_MAP): self.default_callback,
            (NvosDB.COUNTERS_DB, NvosTable.COUNTERS_DB_COUNTERS): self.default_callback,
            (NvosDB.STATE_DB, NvosTable.STATE_DB_IB_PORT_TABLE): self.default_callback,
            (NvosDB.CONFIG_DB, NvosTable.CONFIG_DB_IB_PORT): self.default_callback,
        }

    def _attr_counter_db_key_mapping(self):
        return {
            'logical_port_state': 'SAI_PORT_STAT_INFINIBAND_LOGICAL_STATE',
            'physical_port_state': 'SAI_PORT_STAT_INFINIBAND_PHYSICAL_STATE',
            'ib_speed': 'SAI_PORT_STAT_INFINIBAND_SPEED_OPER',
            'speed': 'SAI_PORT_STAT_INFINIBAND_SPEED_OPER',
            'width': 'SAI_PORT_STAT_INFINIBAND_LANES_OPER',
            'mtu': 'SAI_PORT_STAT_INFINIBAND_MTU_OPER',
            'operational_vls': 'SAI_PORT_STAT_INFINIBAND_VL_OPER',
            'in_pkts': 'SAI_PORT_STAT_INFINIBAND_IF_IN_PKTS_EXT',
            'out_pkts': 'SAI_PORT_STAT_INFINIBAND_IF_OUT_PKTS_EXT',
            'in_errors': 'SAI_PORT_STAT_INFINIBAND_PC_ERR_RCV_F',
            'out_errors': 'SAI_PORT_STAT_INFINIBAND_ERR_XMTCONSTR_F',
            'in_octets': 'SAI_PORT_STAT_INFINIBAND_IF_IN_OCTETS_EXT',
            'out_octets': 'SAI_PORT_STAT_INFINIBAND_IF_OUT_OCTETS_EXT',
            'in_discards': 'SAI_PORT_STAT_INFINIBAND_PC_VL15_DROPPED_F',
            'out_discards': 'SAI_PORT_STAT_INFINIBAND_PC_XMT_DISCARDS_F',
            #error counters
            'port_symbol_errors': 'SAI_PORT_STAT_INFINIBAND_PC_ERR_SYM_F',
            'port_rcv_errors': 'SAI_PORT_STAT_INFINIBAND_PC_ERR_RCV_F',
            'port_vl15_dropped': 'SAI_PORT_STAT_INFINIBAND_PC_VL15_DROPPED_F',
            'port_xmit_wait': 'SAI_PORT_STAT_INFINIBAND_IF_OUT_WAIT',
            'port_link_error_recovery': 'SAI_PORT_STAT_INFINIBAND_ERROR_RECOVERY_EXT',
            'port_link_downed': 'SAI_PORT_STAT_INFINIBAND_LINK_DOWNED_EXT',
            'port_rcv_remote_phy_errors': 'SAI_PORT_STAT_INFINIBAND_RCV_REMOTE_PHY_ERR_EXT',
            'port_rcv_switch_relay_errors': 'SAI_PORT_STAT_INFINIBAND_RCV_SWITCH_RELAY_ERR_EXT',
            'port_rcv_constraints_errors': 'SAI_PORT_STAT_INFINIBAND_RX_CONSTRAINTS_ERR_EXT',
            'port_local_link_integrity_errors': 'SAI_PORT_STAT_INFINIBAND_LOCAL_LINK_INTEGRITY_ERR_EXT',
            'port_excessive_buffer_overrun': 'SAI_PORT_STAT_INFINIBAND_EXCESSIVE_BUFFER_OVERRUN_EXT',
            'port_qp1_dropped': 'SAI_PORT_STAT_INFINIBAND_QP1_DROPPED_EXT',
            #amBER
            'phy_manager_state': 'SAI_PORT_ATTR_PHY_MNGR_FSM_STATE',
            'time_since_last_clear_min': 'SAI_PORT_STAT_INFINIBAND_TIME_SINCE_LAST_CLEAR',
            'raw_ber': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING',
            'effective_ber': 'SAI_PORT_STAT_INFINIBAND_EFFECTIVE_BER_STRING',
            'symbol_ber': 'SAI_PORT_STAT_INFINIBAND_SYMBOL_BER_STRING',
            'zero_hist': 'SAI_PORT_STAT_INFINIBAND_ZERO_HIST',
            'link_down': 'SAI_PORT_STAT_INFINIBAND_LINK_DOWNED_EXT',
            'successful_recovery_events': 'SAI_PORT_STAT_INFINIBAND_SUCCESSFUL_RECOVERY_EVENTS',
            'rs_num_corr_err_bin0': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_0',
            'rs_num_corr_err_bin1': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_1',
            'rs_num_corr_err_bin2': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_2',
            'rs_num_corr_err_bin3': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_3',
            'rs_num_corr_err_bin4': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_4',
            'rs_num_corr_err_bin5': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_5',
            'rs_num_corr_err_bin6': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_6',
            'rs_num_corr_err_bin7': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_7',
            'rs_num_corr_err_bin8': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_8',
            'rs_num_corr_err_bin9': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_9',
            'rs_num_corr_err_bin10': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_10',
            'rs_num_corr_err_bin11': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_11',
            'rs_num_corr_err_bin12': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_12',
            'rs_num_corr_err_bin13': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_13',
            'rs_num_corr_err_bin14': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_14',
            'rs_num_corr_err_bin15': 'SAI_PORT_STAT_INFINIBAND_RS_HIST_15',
            'effective_errors': 'SAI_PORT_STAT_INFINIBAND_EFFECTIVE_ERR',
            'symbol_errors': 'SAI_PORT_STAT_INFINIBAND_SYMBOL_ERR',
            'phy_received_bits': 'SAI_PORT_STAT_INFINIBAND_PHY_RECEIVED_BITS',
            'port_xmit_data_extended': 'SAI_PORT_STAT_INFINIBAND_IF_OUT_OCTETS_EXT',
            'port_rcv_data_extended': 'SAI_PORT_STAT_INFINIBAND_IF_IN_OCTETS_EXT',
            'port_xmit_pkts_extended': 'SAI_PORT_STAT_INFINIBAND_IF_OUT_PKTS_EXT',
            'port_rcv_pkts_extended': 'SAI_PORT_STAT_INFINIBAND_IF_IN_PKTS_EXT',
            'port_uni_cast_xmit_pkts': 'SAI_PORT_STAT_INFINIBAND_UNICAST_XMT_PKTS',
            'port_uni_cast_rcv_pkts': 'SAI_PORT_STAT_INFINIBAND_UNICAST_RCV_PKTS',
            'port_multi_cast_xmit_pkts': 'SAI_PORT_STAT_INFINIBAND_MULTICAST_XMT_PKTS',
            'port_multi_cast_rcv_pkts': 'SAI_PORT_STAT_INFINIBAND_MULTICAST_RCV_PKTS',
            'sync_header_error_counter': 'SAI_PORT_STAT_INFINIBAND_SYNC_HEADER_ERR_COUNTER',
            'port_local_physical_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_LOCAL_PHYSICAL_ERRORS',
            'port_malformed_packet_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_MALFORMED_PACKET_ERR',
            'port_buffer_overrun_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_BUFFER_OVERRUN_ERR',
            'port_dlid_mapping_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_DLID_MAPPING_ERR',
            'port_vl_mapping_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_VL_MAPPING_ERR',
            'port_looping_errors': 'SAI_PORT_STAT_INFINIBAND_PORT_LOOPING_ERR',
            'port_inactive_discards': 'SAI_PORT_STAT_INFINIBAND_PORT_INACTIVE_DISCARD',
            'port_neighbor_mtu_discards': 'SAI_PORT_STAT_INFINIBAND_PORT_NEIGHBOR_MTU_DISCARDS',
            'plr_rcv_codes': 'SAI_PORT_STAT_INFINIBAND_PLR_RCV_CODES',
            'plr_rcv_code_err': 'SAI_PORT_STAT_INFINIBAND_PLR_RCV_CODES_ERR',
            'plr_rcv_uncorrectable_code': 'SAI_PORT_STAT_INFINIBAND_PLR_RCV_UNCORRECTABLE_CODE',
            'plr_xmit_codes': 'SAI_PORT_STAT_INFINIBAND_PLR_XMT_CODES',
            'plr_xmit_retry_codes': 'SAI_PORT_STAT_INFINIBAND_PLR_XMT_RETRY_CODES',
            'plr_xmit_retry_events': 'SAI_PORT_STAT_INFINIBAND_PLR_XMT_RETRY_EVENTS',
            'plr_sync_events': 'SAI_PORT_STAT_INFINIBAND_PLR_SYNC_EVENTS',
            'plr_codes_loss': 'SAI_PORT_STAT_INFINIBAND_PLR_CODES_LOSS',
            'plr_xmit_retry_events_within_t_sec_max': 'SAI_PORT_STAT_INFINIBAND_PLR_XMT_RETRY_EVENTS_WITHIN_TSEC_MAX',
            'rq_general_error': 'SAI_PORT_STAT_INFINIBAND_RQ_GENERAL_ERROR',
            'pd_fsm_state': 'SAI_PORT_ATTR_PD_FSM_STATE',
            'eth_an_fsm_state': 'SAI_PORT_ATTR_ETH_AN_FSM_STATE',
            'ib_phy_fsm_state': 'SAI_PORT_ATTR_IB_PHY_FSM_STATE',
            'phy_hst_fsm_state': 'SAI_PORT_ATTR_PHY_HST_FSM_STATE',
            'psi_fsm_state': 'SAI_PORT_ATTR_PSI_FSM_STATE',
            'phy_manager_link_width_enabled': 'SAI_PORT_ATTR_PHY_MANAGER_LINK_WIDTH_ENABLED',
            'phy_manager_link_proto_enabled': 'SAI_PORT_ATTR_PHY_MANAGER_LINK_PROTO_ENABLED',
            'core_to_phy_link_width_enabled': 'SAI_PORT_ATTR_CORE_TO_PHY_LINK_WIDTH_ENABLED',
            'core_to_phy_link_proto_enabled': 'SAI_PORT_ATTR_CORE_TO_PHY_LINK_PROTO_ENABLED',
            'cable_proto_cap_ext': 'SAI_PORT_ATTR_CABLE_LINK_EXT',
            'loopback_mode': 'SAI_PORT_ATTR_PDDR_LOOPBACK_MODE',
            'retran_mode_request': 'SAI_PORT_ATTR_RETRAN_MODE_REQUEST',
            'retran_mode_active': 'SAI_PORT_ATTR_RETRAN_MODE_ACTIVE',
            'fec_mode_request': 'SAI_PORT_ATTR_FEC_MODE_REQUEST',
            'profile_fec_in_use': 'SAI_PORT_ATTR_PROFILE_FEC_IN_USE',
            'pd_link_width_enabled': 'SAI_PORT_ATTR_PD_LINK_WIDTH_ENABLED',
            'pd_link_speed_enabled': 'SAI_PORT_ATTR_PD_LINK_SPEED_ENABLED',
            'phy_hst_link_width_enabled': 'SAI_PORT_ATTR_PHY_HST_LINK_WIDTH_ENABLED',
            'phy_hst_link_speed_enabled': 'SAI_PORT_ATTR_PHY_HST_LINK_SPEED_ENABLED',
            'raw_ber_ch_1': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_0',
            'raw_ber_ch_2': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_1',
            'raw_ber_ch_3': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_2',
            'raw_ber_ch_4': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_3',
            'raw_ber_ch_5': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_4',
            'raw_ber_ch_6': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_5',
            'raw_ber_ch_7': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_6',
            'raw_ber_ch_8': 'SAI_PORT_STAT_INFINIBAND_RAW_BER_STRING_LANE_7',
            'raw_errors_ch_1': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_0',
            'raw_errors_ch_2': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_1',
            'raw_errors_ch_3': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_2',
            'raw_errors_ch_4': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_3',
            'raw_errors_ch_5': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_4',
            'raw_errors_ch_6': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_5',
            'raw_errors_ch_7': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_6',
            'raw_errors_ch_8': 'SAI_PORT_STAT_INFINIBAND_RAW_ERR_LANE_7'
        }

    def _attr_state_db_key_mapping(self):
        return {
            'supported_ib_speeds': 'speed_admin',
            'supported_widths': 'lanes_admin',
            'max_supported_MTUs': 'mtu_max',
            'ib_subnet': 'ib_subnet',
            'vl_capabilities': 'vl_admin',
            'oper_status': 'oper_status'
        }

    def _attr_config_db_key_mapping(self):
        return {
            'ifname': 'alias',
            'speed_negotiate': 'auto_neg',
            'admin_status': 'admin_status',
            'description': 'description',
            'ifindex': 'index',
        }

    def update_ib_interface_obj(self, ib_interface_obj, data, attr_map):
        if data:
            for attr_name, db_key in attr_map.items():
                try:
                    setattr(ib_interface_obj, attr_name, data.get(db_key, "N/A"))
                except (ValueError, KeyError) as ex:
                    self._logger.debug(
                        'Failed to set attr {} for ib_interface_obj object'.format(attr_name))
            pass

    def _create_ib_interface_obj(self, counter_db_data, state_db_data, config_db_data):
        ib_interface_obj = IbInterface()
        self.update_ib_interface_obj(ib_interface_obj, counter_db_data, self._attr_counter_db_key_mapping())
        self.update_ib_interface_obj(ib_interface_obj, state_db_data, self._attr_state_db_key_mapping())
        self.update_ib_interface_obj(ib_interface_obj, config_db_data, self._attr_config_db_key_mapping())
        try:
            ib_interface_obj.plr_bw_loss_percent = str(float(ib_interface_obj.plr_codes_loss) / PLR_BW_LOSS_PERCENT_DIVISOR)
        except Exception:
            ib_interface_obj.plr_bw_loss_percent = 'N/A'
        return ib_interface_obj

    def process_fullstate(self, data):
        port_map = self.data_get(data, NvosDB.COUNTERS_DB,
                                 NvosTable.COUNTERS_DB_COUNTERS_PORT_NAME_MAP)  # ifname -> oid
        if not port_map:
            self._logger.debug('Failed to retrieve {} from DB {}'.format(NvosTable.COUNTERS_DB_COUNTERS_PORT_NAME_MAP,
                                                                         NvosDB.COUNTERS_DB))
            return
        for ifname, oid in port_map.items():
            counter_db_record = self.data_get(data, NvosDB.COUNTERS_DB, NvosTable.COUNTERS_DB_COUNTERS).get(oid)
            state_db_record = self.data_get(data, NvosDB.STATE_DB, NvosTable.STATE_DB_IB_PORT_TABLE).get(ifname)
            config_db_record = self.data_get(data, NvosDB.CONFIG_DB, NvosTable.CONFIG_DB_IB_PORT).get(ifname)

            ib_interface_record = self._create_ib_interface_obj(counter_db_record, state_db_record,
                                                                config_db_record)
            if ib_interface_record:
                self._logger.debug(ib_interface_record)
                ib_interface_record.save()


def process_counters(*args, **kwargs):
    logger = kwargs.get('logger')
    render.process(logger)


render = IbInterfaceHandler()
